/*-------------------------------------------------------------------------
 *
 * tuplestore.h
 *	  Generalized routines for temporary tuple storage.
 *
 * This module handles temporary storage of tuples for purposes such
 * as Materialize nodes, hashjoin batch files, etc.  It is essentially
 * a dumbed-down version of tuplesort.c; it does no sorting of tuples
 * but can only store a sequence of tuples and regurgitate it later.
 * A temporary file is used to handle the data if it exceeds the
 * space limit specified by the caller.
 *
 * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * $Id: tuplestore.h,v 1.6 2001/11/05 17:46:36 momjian Exp $
 *
 *-------------------------------------------------------------------------
 */
#ifndef TUPLECONN_H
#define TUPLECONN_H

#include "access/htup.h"
#include "nodes/nodes.h"
#include "executor/executor.h"

/* Tuplestorestate is an opaque type whose details are not known outside
 * tuplestore.c.
 */
typedef struct Tupleconnstate Tupleconnstate;

/*
 * Currently we only need to store HeapTuples, but it would be easy
 * to support the same behavior for IndexTuples and/or bare Datums.
 */

extern Tupleconnstate *tupleconn_begin_heap(
					  int maxKBytes, TupleDesc tupDesc, Oid connOp, List *priorsList,
            int levelColNum,double rows,bool useHash);

extern void tupleconn_puttuple(Tupleconnstate *state, void *tuple,int head);

extern void tupleconn_donestoring(Tupleconnstate *state);

extern void *tupleconn_gettuple(Tupleconnstate *state,
					bool *should_free);

#define tupleconn_getheaptuple(state, should_free) \
	((HeapTuple) tupleconn_gettuple(state, should_free))

extern void tupleconn_end(Tupleconnstate *state);

extern void tupleconn_performconn(Tupleconnstate *state, ExprState *parentExpr, ExprState *childExpr, ExprContext *econtext);

/*
 * These routines may only be called if randomAccess was specified 'true'.
 * Likewise, backwards scan in gettuple/getdatum is only allowed if
 * randomAccess was specified.
 */

extern void tupleconn_rescan(Tupleconnstate *state);
extern void tupleconn_markpos(Tupleconnstate *state);
extern void tupleconn_restorepos(Tupleconnstate *state);

#endif   /* TUPLECONN_H */
